@@ -10,12 +10,10 @@
 # FOR A PARTICULAR PURPOSE
 #
 ##############################################################################
-__doc__="""Cacheable object and cache management base classes.
-
-$Id: Cache.py,v 1.10.68.1 2003/11/17 22:34:07 tseaver Exp $"""
-
-__version__='$Revision: 1.10.68.1 $'[11:-2]
+"""Cacheable object and cache management base classes.
 
+$Id: Cache.py 24764 2004-05-17 06:13:48Z philikon $
+"""
 import time, sys
 import Globals
 from Globals import DTMLFile
@@ -92,7 +90,6 @@ class Cacheable:
         (ViewManagementScreensPermission,
          ('ZCacheable_manage',
           'ZCacheable_invalidate',
-          'ZCacheable_isMethod',
           'ZCacheable_enabled',
           'ZCacheable_getManagerId',
           'ZCacheable_getManagerIds',
