@@ -12,7 +12,7 @@
 ##############################################################################
 """Object Manager
 
-$Id: ObjectManager.py 30356 2005-05-15 15:56:18Z yuppie $
+$Id: ObjectManager.py 41393 2006-01-21 14:14:12Z sidnei $
 """
 
 import sys, fnmatch, copy, os, re
@@ -49,7 +49,7 @@ customImporters={
     XMLExportImport.magic: XMLExportImport.importXML,
     }
 
-bad_id=re.compile(r'[^a-zA-Z0-9-_~,.$\(\)# ]').search #TS
+bad_id=re.compile(r'[^a-zA-Z0-9-_~,.$\(\)# @]').search
 
 def checkValidId(self, id, allow_dup=0):
     # If allow_dup is false, an error will be raised if an object
@@ -261,6 +261,20 @@ class ObjectManager(
             raise AttributeError, id
         return default
 
+    def hasObject(self, id):
+        """Indicate whether the folder has an item by ID.
+
+        This doesn't try to be more intelligent than _getOb, and doesn't
+        consult _objects (for performance reasons). The common use case
+        is to check that an object does *not* exist.
+        """
+        if (id in ('.', '..') or
+            id.startswith('_') or
+            id.startswith('aq_') or
+            id.endswith('__')):
+            return False
+        return getattr(aq_base(self), id, None) is not None
+
     def _setObject(self, id, object, roles=None, user=None, set_owner=1):
         v=self._checkId(id)
         if v is not None: id=v
@@ -500,7 +514,7 @@ class ObjectManager(
             obj_ids.sort()
             for id in obj_ids:
                 o=self._getOb(id)
-                if hasattr(o, 'isPrincipiaFolderish') and \
+                if hasattr(aq_base(o), 'isPrincipiaFolderish') and \
                    o.isPrincipiaFolderish:
                     r.append(o)
         return r
@@ -615,7 +629,7 @@ class ObjectManager(
                 break
             ob=ob.aq_parent
 
-        files=self.objectItems()
+        files = list(self.objectItems())
 
         # recursive ride through all subfolders (ls -R) (ajung)
 
@@ -623,15 +637,10 @@ class ObjectManager(
 
             all_files = copy.copy(files)
             for f in files:
-                if f[1].meta_type == "Folder":
+                if hasattr(aq_base(f[1]), 'isPrincipiaFolderish') and f[1].isPrincipiaFolderish:
                     all_files.extend(findChildren(f[1]))
-                else:
-                    all_files.append(f)
-
             files = all_files
 
-        files = list(files)
-
         # Perform globbing on list of files (ajung)
 
         globbing = REQUEST.environ.get('GLOBBING','')
@@ -709,12 +718,12 @@ def findChildren(obj,dirname=''):
     find all children of an object (ajung)
     """
 
-    lst =[]
-    for name,child in obj.objectItems():
-        if child.meta_type=="Folder":
-            lst.extend(findChildren(child,dirname+ obj.id + '/'))
+    lst = []
+    for name, child in obj.objectItems():
+        if hasattr(aq_base(child), 'isPrincipiaFolderish') and child.isPrincipiaFolderish:
+            lst.extend(findChildren(child, dirname + obj.id + '/'))
         else:
-            lst.append( (dirname + obj.id + "/" + name,child) )
+            lst.append((dirname + obj.id + "/" + name, child))
 
     return lst
 
