@@ -1,18 +1,16 @@
 import unittest
 import cStringIO
-from mimetools import Message
-from multifile import MultiFile
 
-from AccessControl import SecurityManager, Unauthorized
+import transaction
+from AccessControl import SecurityManager
 from AccessControl.SecurityManagement import newSecurityManager
 from AccessControl.SecurityManagement import noSecurityManager
-from Acquisition import Implicit
 from Acquisition import aq_base
+from Acquisition import Implicit
 from OFS.Application import Application
 from OFS.Folder import manage_addFolder
 from OFS.Image import manage_addFile
 from Testing.makerequest import makerequest
-from webdav.common import rfc1123_date
 
 
 ADD_IMAGES_AND_FILES = 'Add images and files'
@@ -86,17 +84,17 @@ class CopySupportTestBase(unittest.TestCase):
             # Hack, we need a _p_mtime for the file, so we make sure that it
             # has one. We use a subtransaction, which means we can rollback
             # later and pretend we didn't touch the ZODB.
-            get_transaction().commit()
+            transaction.commit()
         except:
             self.connection.close()
             raise
-        get_transaction().begin()
+        transaction.begin()
 
         return self.app._getOb( 'folder1' ), self.app._getOb( 'folder2' )
 
     def _cleanApp( self ):
 
-        get_transaction().abort()
+        transaction.abort()
         self.app._p_jar.sync()
         self.connection.close()
         del self.app
@@ -280,9 +278,12 @@ class _SensitiveSecurityPolicy:
         self._lambdas = ( validate_lambda, checkPermission_lambda )
 
     def validate( self, *args, **kw ):
-        if self._lambdas[ 0 ]( *args, **kw ):
-            return 1
-        raise Unauthorized
+        from zExceptions import Unauthorized
+
+        allowed = self._lambdas[ 0 ]( *args, **kw )
+        if not allowed:
+            raise Unauthorized
+        return 1
 
     def checkPermission( self, *args, **kw ) :
         return self._lambdas[ 1 ]( *args, **kw )
@@ -330,7 +331,7 @@ class TestCopySupportSecurity( CopySupportTestBase ):
         except CopyError, e:
 
             if ce_regex is not None:
-                
+
                 pattern = re.compile( ce_regex, re.DOTALL )
                 if pattern.search( e ) is None:
                     self.fail( "Paste failed; didn't match pattern:\n%s" % e )
@@ -344,7 +345,7 @@ class TestCopySupportSecurity( CopySupportTestBase ):
         else:
             self.fail( "Paste allowed unexpectedly." )
 
-    def _initPolicyAndUser( self    
+    def _initPolicyAndUser( self
                           , a_lambda=None
                           , v_lambda=None
                           , c_lambda=None
@@ -402,7 +403,7 @@ class TestCopySupportSecurity( CopySupportTestBase ):
                                    )
 
     def test_copy_cant_create_target_metatype_not_supported( self ):
-        
+
         from OFS.CopySupport import CopyError
 
         folder1, folder2 = self._initFolders()
@@ -433,7 +434,7 @@ class TestCopySupportSecurity( CopySupportTestBase ):
         self.failUnless( 'file' in folder2.objectIds() )
 
     def test_move_cant_read_source( self ):
-        
+
         from OFS.CopySupport import CopyError
 
         folder1, folder2 = self._initFolders()
@@ -453,7 +454,7 @@ class TestCopySupportSecurity( CopySupportTestBase ):
                                    )
 
     def test_move_cant_create_target_metatype_not_supported( self ):
-        
+
         from OFS.CopySupport import CopyError
 
         folder1, folder2 = self._initFolders()
@@ -468,16 +469,16 @@ class TestCopySupportSecurity( CopySupportTestBase ):
                                    )
 
     def test_move_cant_create_target_metatype_not_allowed( self ):
-        
+
         from OFS.CopySupport import CopyError
 
         folder1, folder2 = self._initFolders()
         folder2.all_meta_types = FILE_META_TYPES
 
-        def _no_manage_addFile( a, c, n, v, *args, **kw ):
-            return n != 'manage_addFile'
+        def _no_add_images_and_files(permission, object, context):
+            return permission != ADD_IMAGES_AND_FILES
 
-        self._initPolicyAndUser( v_lambda=_no_manage_addFile )
+        self._initPolicyAndUser( c_lambda=_no_add_images_and_files )
 
         cookie = folder1.manage_cutObjects( ids=( 'file', ) )
         self._assertCopyErrorUnauth( folder2.manage_pasteObjects
@@ -487,7 +488,7 @@ class TestCopySupportSecurity( CopySupportTestBase ):
                                    )
 
     def test_move_cant_delete_source( self ):
-        
+
         from OFS.CopySupport import CopyError
         from AccessControl.Permissions import delete_objects as DeleteObjects
 
@@ -514,8 +515,5 @@ def test_suite():
     suite.addTest( unittest.makeSuite( TestCopySupportSecurity ) )
     return suite
 
-def main():
-    unittest.TextTestRunner().run(test_suite())
-
 if __name__ == '__main__':
-    main()
+    unittest.main(defaultTest='test_suite')
