@@ -12,7 +12,7 @@
 ##############################################################################
 """ ZCatalog product
 
-$Id: ZCatalog.py 33280 2005-07-12 15:02:15Z tiran $
+$Id: ZCatalog.py 39977 2005-11-08 15:19:40Z chrisw $
 """
 
 from warnings import warn
@@ -35,6 +35,7 @@ from ZODB.POSException import ConflictError
 import transaction
 from Products.PluginIndexes.common.PluggableIndex \
      import PluggableIndexInterface
+from Products.PluginIndexes.interfaces import IPluggableIndex
 from Products.PluginIndexes.TextIndex import Splitter
 from zLOG import LOG
 from zope.interface import implements
@@ -295,7 +296,8 @@ class ZCatalog(Folder, Persistent, Implicit):
 
             p = paths[i]
             obj = self.resolve_path(p)
-            if not obj:
+            
+            if not obj and hasattr(self, 'REQUEST'):
                 obj = self.resolve_url(p, self.REQUEST)
             if obj is not None:
                 try:
@@ -586,7 +588,7 @@ class ZCatalog(Folder, Persistent, Implicit):
             # we should commit a subtransaction if our threshhold is
             # exceeded within the boundaries of the current transaction.
             if self._v_total > self.threshold:
-                transaction.savepoint()
+                transaction.savepoint(optimistic=True)
                 self._p_jar.cacheGC()
                 self._v_total = 0
                 if pghandler:
@@ -932,7 +934,7 @@ class ZCatalog(Folder, Persistent, Implicit):
             bases = [str(name) for name in idx.__class__.__bases__]
             found = False
 
-            if idx.meta_type  == 'PathIndex':
+            if idx.meta_type == 'PathIndex':
                 found = True
             else:
                 for base in bases:
@@ -957,6 +959,7 @@ class ZCatalog(Folder, Persistent, Implicit):
 
                 if indexed_attrs:
                     setattr(new_idx, 'indexed_attrs', indexed_attrs)
+
                 if idx.meta_type == 'DateRangeIndex':
                     setattr(new_idx, '_since_field',  since_field)
                     setattr(new_idx, '_until_field', until_field)
@@ -983,7 +986,7 @@ class ZCatalog(Folder, Persistent, Implicit):
         # this interface by that name.  Bleah
 
         products = ObjectManager.all_meta_types(self, interfaces=(
-            PluggableIndexInterface,))
+            PluggableIndexInterface, IPluggableIndex))
 
         p = None
 
