@@ -13,7 +13,7 @@
 __doc__='''Generic Database adapter'''
 
 
-__version__='$Revision: 1.110.12.7 $'[11:-2]
+__version__='$Revision: 1.116 $'[11:-2]
 
 import OFS.SimpleItem, Aqueduct, RDB, re
 import DocumentTemplate, marshal, md5, base64, Acquisition, os
@@ -100,7 +100,7 @@ class DA(
     __ac_permissions__=(
         ('View management screens',
          (
-        'manage_main', 'index_html',
+        'index_html',
         'manage_advancedForm', 'PrincipiaSearchSource', 'document_src'
         )),
         ('Change Database Methods',
@@ -277,7 +277,7 @@ class DA(
         return '<params>%s</params>\n%s' % (self.arguments_src, self.src)
 
     def get_size(self): return len(self.document_src())
-    
+
     def PUT(self, REQUEST, RESPONSE):
         """Handle put requests"""
         self.dav__init(REQUEST, RESPONSE)
@@ -332,7 +332,7 @@ class DA(
                 '<dtml-var manage_tabs>\n<hr>\n%s\n\n'
                 '<hr><strong>SQL used:</strong><br>\n<pre>\n%s\n</pre>\n<hr>\n'
                 '</body></html>'
-                % (r, html_quote(src)))
+                % (r,html_quote(src)))
 
             report=apply(report,(self,REQUEST),{self.id:result})
 
