@@ -13,7 +13,7 @@
 ##############################################################################
 """Interpreter for a pre-compiled TAL program.
 
-$Id: TALInterpreter.py 37696 2005-08-04 14:22:37Z yuppie $
+$Id: TALInterpreter.py 38760 2005-10-05 00:14:42Z tseaver $
 """
 import cgi
 import sys
@@ -506,7 +506,8 @@ class TALInterpreter:
             return
         if isinstance(text, MessageID):
             # Translate this now.
-            text = self.engine.translate(text.domain, text, text.mapping)
+            text = self.engine.translate(text.domain, text, text.mapping,
+                                         default=text.default)
         s = escape(text)
         self._stream_write(s)
         i = s.rfind('\n')
@@ -542,8 +543,9 @@ class TALInterpreter:
             # evaluate() does not do any I18n, so we do it here. 
             if isinstance(value, MessageID):
                 # Translate this now.
+                # XXX
                 value = self.engine.translate(value.domain, value,
-                                              value.mapping)
+                                              value.mapping, value.default)
 
             if not structure:
                 value = cgi.escape(ustr(value))
