@@ -12,7 +12,7 @@
 ##############################################################################
 """ Basic ZPublisher request management.
 
-$Id: BaseRequest.py,v 1.51.2.4 2004/12/02 16:49:28 andreasjung Exp $
+$Id: BaseRequest.py 41707 2006-02-19 18:53:08Z shh $
 """
 
 from urllib import quote
@@ -31,12 +31,19 @@ try:
         def manage_property_types(self):
             return type_converters.keys()
 
-except:
+except ImportError:
     class RequestContainer:
         __roles__=None
         def __init__(self,**kw):
             for k,v in kw.items(): self.__dict__[k]=v
 
+try:
+    from AccessControl.ZopeSecurityPolicy import getRoles
+except ImportError:
+    def getRoles(container, name, value, default):
+        return getattr(value, '__roles__', default)
+
+
 _marker=[]
 class BaseRequest:
     """Provide basic ZPublisher request management
@@ -143,6 +150,9 @@ class BaseRequest:
     def has_key(self,key):
         return self.get(key, _marker) is not _marker
 
+    def __contains__(self, key):
+        return self.has_key(key)
+    
     def keys(self):
         keys = {}
         keys.update(self.common)
@@ -222,14 +232,14 @@ class BaseRequest:
         object=parents[-1]
         del parents[:]
 
-        roles = getattr(object, '__roles__', UNSPECIFIED_ROLES)
+        roles = getRoles(None, None, object, UNSPECIFIED_ROLES)
 
         # if the top object has a __bobo_traverse__ method, then use it
         # to possibly traverse to an alternate top-level object.
         if hasattr(object,'__bobo_traverse__'):
             try:
                 object=object.__bobo_traverse__(request)
-                roles =getattr(object, '__roles__', UNSPECIFIED_ROLES)
+                roles = getRoles(None, None, object, UNSPECIFIED_ROLES)
             except: pass
 
         if not path and not method:
@@ -249,6 +259,9 @@ class BaseRequest:
         request['TraversalRequestNameStack'] = request.path = path
         request['ACTUAL_URL'] = request['URL'] + quote(browser_path)
 
+        # Set the posttraverse for duration of the traversal here
+        self._post_traverse = post_traverse = []
+
         entry_name = ''
         try:
             # We build parents in the wrong order, so we
@@ -285,9 +298,9 @@ class BaseRequest:
                     entry_name = method
                     method = 'index_html'
                 else:
-                    if (hasattr(object, '__call__') and
-                        hasattr(object.__call__,'__roles__')):
-                        roles=object.__call__.__roles__
+                    if (hasattr(object, '__call__')):
+                        roles = getRoles(object, '__call__', object.__call__,
+                                         roles)
                     if request._hacked_path:
                         i=URL.rfind('/')
                         if i > 0: response.setBase(URL[:i])
@@ -368,13 +381,9 @@ class BaseRequest:
                         "The object at %s is not publishable." % URL
                         )
 
-                r = getattr(subobject, '__roles__', UNSPECIFIED_ROLES)
-                if r is not UNSPECIFIED_ROLES:
-                    roles = r
-                elif not got:
-                    # We got the subobject as an attribute, not an item,
-                    # so we should check "next to it" for __roles__.
-                    roles = getattr(object, entry_name+'__roles__', roles)
+                roles = getRoles(
+                    object, (not got) and entry_name or None, subobject,
+                    roles)
 
                 # Promote subobject to object
                 object=subobject
@@ -383,6 +392,9 @@ class BaseRequest:
                 steps.append(entry_name)
         finally:
             parents.reverse()
+ 
+        # After traversal post traversal hooks aren't available anymore
+        del self._post_traverse
 
         request['PUBLISHED'] = parents.pop(0)
 
@@ -456,8 +468,31 @@ class BaseRequest:
         # Remove http request method from the URL.
         request['URL']=URL
 
+        # Run post traversal hooks
+        if post_traverse:
+            result = exec_callables(post_traverse)
+            if result is not None:
+                object = result
+
         return object
 
+    def post_traverse(self, f, args=()):
+        """Add a callable object and argument tuple to be post-traversed.
+        
+        If traversal and authentication succeed, each post-traversal
+        pair is processed in the order in which they were added.
+        Each argument tuple is passed to its callable.  If a callable
+        returns a value other than None, no more pairs are processed,
+        and the return value replaces the traversal result.
+        """
+        try:
+            pairs = self._post_traverse
+        except AttributeError:
+            raise RuntimeError, ('post_traverse() may only be called '
+                                 'during publishing traversal.')
+        else:
+            pairs.append((f, tuple(args)))
+
     retry_count=0
     def supports_retry(self): return 0
 
@@ -466,8 +501,13 @@ class BaseRequest:
         """
         self._held=self._held+(object,)
 
-
-
+def exec_callables(callables):
+    result = None
+    for (f, args) in callables:
+        # Don't catch exceptions here. And don't hide them anyway.
+        result = f(*args)
+        if result is not None:
+            return result
 
 def old_validation(groups, request, auth,
                    roles=UNSPECIFIED_ROLES):
@@ -526,15 +566,22 @@ def old_validation(groups, request, auth,
 # Zope 3 by then :)
 
 import types
+import sys
 
 itypes = {}
 for name in ('NoneType', 'IntType', 'LongType', 'FloatType', 'StringType',
              'BufferType', 'TupleType', 'ListType', 'DictType', 'XRangeType',
              'SliceType', 'EllipsisType', 'UnicodeType', 'CodeType',
-             'TracebackType', 'FrameType', 'DictProxyType'):
+             'TracebackType', 'FrameType', 'DictProxyType', 'BooleanType',
+             'ComplexType'):
     if hasattr(types, name):
         itypes[getattr(types, name)] = 0
 
+# Python 2.4 no longer maintains the types module.
+if sys.version_info >= (2, 4):
+    itypes[set] = 0
+    itypes[frozenset] = 0
+
 def typeCheck(obj, deny=itypes):
     # Return true if its ok to publish the type, false otherwise.
     return deny.get(type(obj), 1)
