@@ -12,7 +12,7 @@
 ##############################################################################
 '''CGI Response Output formatter
 
-$Id: HTTPResponse.py 28683 2004-12-22 15:49:05Z andreasjung $'''
+$Id: HTTPResponse.py 41457 2006-01-27 01:31:00Z tseaver $'''
 __version__ = '$Revision: 1.81 $'[11:-2]
 
 import types, os, sys, re
@@ -26,6 +26,8 @@ from cgi import escape
 
 nl2sp = maketrans('\n',' ')
 
+# This may get overwritten during configuration
+default_encoding = 'iso-8859-15'
 
 # Enable APPEND_TRACEBACKS to make Zope append tracebacks like it used to,
 # but a better solution is to make standard_error_message display error_tb.
@@ -444,7 +446,7 @@ class HTTPResponse(BaseResponse):
                 encoding = match.group(1)
                 return body.encode(encoding)
         # Use the default character encoding
-        return body.encode('iso-8859-15','replace')
+        return body.encode(default_encoding,'replace')
 
     def setBase(self,base):
         """Set the base URL for the returned document.
@@ -544,7 +546,7 @@ class HTTPResponse(BaseResponse):
         Sets an HTTP return header "name" with value "value",
         appending it following a comma if there was a previous value
         set for the header. '''
-        name = str(name)
+        name = str(name).lower()
         value = str(value)
 
         headers = self.headers
@@ -597,42 +599,43 @@ class HTTPResponse(BaseResponse):
     def _error_html(self,title,body):
         # XXX could this try to use standard_error_message somehow?
         return ("""\
-<TABLE BORDER="0" WIDTH="100%">
-<TR VALIGN="TOP">
+<table border="0" width="100%">
+<tr valign="top">
 
-<TD WIDTH="10%" ALIGN="CENTER">
+<td width="10%" align="center">
 &nbsp;
-</TD>
+</td>
 
-<TD WIDTH="90%">
-  <H2>Site Error</H2>
-  <P>An error was encountered while publishing this resource.
-  </P>""" + \
+<td width="90%">
+  <h2>Site Error</h2>
+  <p>An error was encountered while publishing this resource.
+  </p>""" + \
   """
-  <P><STRONG>%s</STRONG></P>
+  <p><strong>%s</strong></p>
 
   %s""" %(title,body) + \
   """
-  <HR NOSHADE>
+  <hr noshade="noshade"/>
 
-  <P>Troubleshooting Suggestions</P>
+  <p>Troubleshooting Suggestions</p>
 
-  <UL>
-  <LI>The URL may be incorrect.</LI>
-  <LI>The parameters passed to this resource may be incorrect.</LI>
-  <LI>A resource that this resource relies on may be
-      encountering an error.</LI>
-  </UL>
+  <ul>
+  <li>The URL may be incorrect.</li>
+  <li>The parameters passed to this resource may be incorrect.</li>
+  <li>A resource that this resource relies on may be
+      encountering an error.</li>
+  </ul>
 
-  <P>For more detailed information about the error, please
+  <p>For more detailed information about the error, please
   refer to error log.
-  </P>
+  </p>
 
-  <P>If the error persists please contact the site maintainer.
+  <p>If the error persists please contact the site maintainer.
   Thank you for your patience.
-  </P>
-</TD></TR>
-</TABLE>""")
+  </p>
+</td></tr>
+</table>""")
+
 
     def notFoundError(self,entry='Unknown'):
         self.setStatus(404)
@@ -649,7 +652,7 @@ class HTTPResponse(BaseResponse):
         raise NotFound, self._error_html(
             "Debugging Notice",
             "Zope has encountered a problem publishing your object.<p>"
-            "\n%s" % entry)
+            "\n%s</p>" % entry)
 
     def badRequestError(self,name):
         self.setStatus(400)
@@ -663,7 +666,7 @@ class HTTPResponse(BaseResponse):
             "The parameter, <em>%s</em>, " % name +
             "was omitted from the request.<p>" +
             "Make sure to specify all required parameters, " +
-            "and try the request again."
+            "and try the request again.</p>"
             )
 
     def _unauthorized(self):
@@ -676,9 +679,9 @@ class HTTPResponse(BaseResponse):
         m = "<strong>You are not authorized to access this resource.</strong>"
         if self.debug_mode:
             if self._auth:
-                m = m + '<p>\nUsername and password are not correct.'
+                m = m + '<p>\nUsername and password are not correct.</p>'
             else:
-                m = m + '<p>\nNo Authorization header found.'
+                m = m + '<p>\nNo Authorization header found.</p>'
         raise Unauthorized, m
 
     def exception(self, fatal=0, info=None,
@@ -768,7 +771,7 @@ class HTTPResponse(BaseResponse):
         if fatal and t is SystemExit and v.code == 0:
             body = self.setBody(
                 (str(t),
-                 'Zope has exited normally.<p>' + self._traceback(t, v, tb)),
+                 'Zope has exited normally.<p>' + self._traceback(t, v, tb) + '</p>'),
                 is_error=1)
         else:
             try:
@@ -779,7 +782,8 @@ class HTTPResponse(BaseResponse):
                 body = self.setBody(
                     (str(t),
                      'Sorry, a site error occurred.<p>'
-                     + self._traceback(t, v, tb)),
+                     + self._traceback(t, v, tb)
+                     + '</p>'),
                     is_error=1)
             elif self.isHTML(b):
                 # error is an HTML document, not just a snippet of html
