@@ -13,14 +13,12 @@
 
 """WebDAV support - resource objects.
 
-$Id: Resource.py,v 1.55.10.10 2005/06/21 15:27:30 sidnei Exp $
+$Id: Resource.py 30876 2005-06-21 15:33:34Z sidnei $
 """
 
-import os
 import sys
 import mimetypes
-import time
-from urllib import quote, unquote
+from urllib import unquote
 
 import Globals
 import ExtensionClass
@@ -67,13 +65,12 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
         # Init expected HTTP 1.1 / WebDAV headers which are not
         # currently set by the base response object automagically.
         #
-        # Also, we sniff for a ZServer response object, because we don't
+        # We sniff for a ZServer response object, because we don't
         # want to write duplicate headers (since ZS writes Date
         # and Connection itself).
         if not hasattr(response, '_server_version'):
             response.setHeader('Connection', 'close')
             response.setHeader('Date', rfc1123_date(), 1)
-        # XXMSXX response.setHeader('MS-Author-Via', 'DAV')
 
         # HTTP Range support
         if HTTPRangeInterface.isImplementedBy(self):
@@ -323,7 +320,6 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
             raise BadRequest, 'Invalid Destination header'
 
         name = path.pop()
-        parent_path = '/'.join(path)
 
         oflag=REQUEST.get_header('Overwrite', 'F').upper()
         if not oflag in ('T', 'F'):
@@ -578,7 +574,6 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
         """Remove an existing lock on a resource."""
         self.dav__init(REQUEST, RESPONSE)
         security = getSecurityManager()
-        user = security.getUser()
         token = REQUEST.get_header('Lock-Token', '')
         url = REQUEST['URL']
         token = tokenFinder(token)
