@@ -14,7 +14,7 @@
 #
 ##############################################################################
 """
-test.py [-abcCdDfgGhLmprtTuv] [modfilter [testfilter]]
+test.py [-abcCdDfgGhLmpqrtTuv] [modfilter [testfilter]]
 
 Find and run tests written using the unittest module.
 
@@ -104,6 +104,9 @@ named .testinfo, it will not be searched for tests.  Really.)
     Keep running the selected tests in a loop.  You may experience
     memory leakage.
 
+--nowarnings
+    Install a filter to suppress warnings emitted by code.
+
 -t
     Time the individual tests and print a list of the top 50, sorted from
     longest to shortest.
@@ -121,11 +124,32 @@ named .testinfo, it will not be searched for tests.  Really.)
     utility writes coverage files to a directory named `coverage' that
     is parallel to `build'.  It also prints a summary to stdout.
 
+--coverage
+    Use the coverage.py module from Gareth Rees
+    (http://www.nedbatchelder.com/code/modules/coverage.html) to collect data
+    for code coverage.  This will output trace data in a file called
+    '.coverage'. You will need to call coverage.py after the test run to
+    analyse the data (-r for a line count report, -a for annotated file).
+
+--profile
+    Use the profile module from the standard library to collect profiling data.
+    This will output data in a file called '.profile'. You will need to use the
+    pstats module from the standard library after the test run to analyse the
+    data.
+
+--hotshot
+    Use the hotshot module from the standard library to collect profiling data.
+    This will output data in a file called '.hotshot'. You will need to use the
+    hotshot.pstats module from the standard library after the test run to
+    analyse the data. You may also use the hotshot2cg script from the
+    KCacheGrind sources to create data suitable for analysis by KCacheGrind.
+
 -v
+-q
     Verbose output.  With one -v, unittest prints a dot (".") for each
     test run.  With -vv, unittest prints the name of each test (for
-    some definition of "name" ...).  With no -v, unittest is silent
-    until the end of the run, except when errors occur.
+    some definition of "name" ...).  With no -v (or with -q), unittest is
+    silent until the end of the run, except when errors occur.
 
 -u
 -m
@@ -586,6 +610,8 @@ class TrackRefs:
 def runner(files, test_filter, debug):
     runner = ImmediateTestRunner(verbosity=VERBOSE, debug=debug,
                                  progress=progress)
+    result = runner.result
+    
     suite = unittest.TestSuite()
     for file in files:
         s = get_suite(file, runner.result)
@@ -602,6 +628,8 @@ def runner(files, test_filter, debug):
                 print "Wrote timing data to", timesfn
         if timetests:
             r.print_times(sys.stdout, timetests)
+        numbad = len(result.failures) + len(result.errors)
+        return numbad
     except:
         if debugger:
             pdb.post_mortem(sys.exc_info()[2])
@@ -619,7 +647,7 @@ def walk_with_symlinks(path, visit, arg):
     except os.error:
         return
     visit(arg, path, names)
-    exceptions = (os.curdir, os.pardir)
+    exceptions = (os.curdir, os.pardir, 'var')
     for name in names:
         if name not in exceptions:
             name = os.path.join(path, name)
@@ -683,7 +711,7 @@ def main(module_filter, test_filter, libdir):
             rc = sys.gettotalrefcount()
             track = TrackRefs()
         while True:
-            runner(files, test_filter, debug)
+            numbad = runner(files, test_filter, debug)
             gc.collect()
             if gc.garbage:
                 print "GARBAGE:", len(gc.garbage), gc.garbage
@@ -694,8 +722,9 @@ def main(module_filter, test_filter, libdir):
                 print "totalrefcount=%-8d change=%-6d" % (rc, rc - prev)
                 track.update()
     else:
-        runner(files, test_filter, debug)
+        numbad = runner(files, test_filter, debug)
 
+    return numbad
 
 def configure_logging():
     """Initialize the logging module."""
@@ -739,6 +768,7 @@ def process_args(argv=None):
     global test_dir
     global config_file
     global import_testing
+    global no_warnings
 
     if argv is None:
         argv = sys.argv
@@ -749,6 +779,9 @@ def process_args(argv=None):
     LOOP = False
     GUI = False
     TRACE = False
+    COVERAGE = False
+    PROFILE = False
+    HOTSHOT = False
     REFCOUNT = False
     debug = False # Don't collect test results; simply let tests crash
     debugger = False
@@ -766,12 +799,14 @@ def process_args(argv=None):
     test_dir = None
     config_file = None
     import_testing = False
+    no_warnings = False
 
     try:
-        opts, args = getopt.getopt(argv[1:], "a:bcC:dDfg:G:hLmprtTuv",
+        opts, args = getopt.getopt(argv[1:], "a:bcC:dDfg:G:hLmpqrtTuv",
                                    ["all", "help", "libdir=", "times=",
-                                    "keepbytecode", "dir=", 
-                                    "config-file=", "import-testing"])
+                                    "keepbytecode", "nowarnings", "dir=",
+                                    "config-file=", "import-testing",
+                                    "coverage", "profile", "hotshot"])
     except getopt.error, msg:
         print msg
         print "Try `python %s -h' for more information." % argv[0]
@@ -823,6 +858,12 @@ def process_args(argv=None):
                 print "-r ignored, because it needs a debug build of Python"
         elif k == "-T":
             TRACE = True
+        elif k == "--coverage":
+            COVERAGE = True
+        elif k == "--profile":
+            PROFILE = True
+        elif k == "--hotshot":
+            HOTSHOT = True
         elif k == "-t":
             if not timetests:
                 timetests = 50
@@ -830,6 +871,8 @@ def process_args(argv=None):
             GUI = 1
         elif k == "-v":
             VERBOSE += 1
+        elif k == "-q":
+            VERBOSE = 0
         elif k == "--times":
             try:
                 timetests = int(v)
@@ -842,6 +885,12 @@ def process_args(argv=None):
             config_file = v
         elif k == '--import-testing':
             import_testing = True
+        elif k == '--nowarnings':
+            no_warnings = True
+
+    if no_warnings:
+        import warnings
+        warnings.simplefilter('ignore', Warning, append=1)
 
     if gcthresh is not None:
         if gcthresh == 0:
@@ -903,6 +952,29 @@ def process_args(argv=None):
                           globals=globals(), locals=vars())
             r = tracer.results()
             r.write_results(show_missing=True, summary=True, coverdir=coverdir)
+
+        elif COVERAGE:
+            try:
+                from coverage import the_coverage
+            except:
+                print "You need to install coverage.py from "
+                print "http://www.nedbatchelder.com/code/modules/coverage.html"
+                sys.exit()
+            the_coverage.start()
+            main(module_filter, test_filter, libdir)
+
+        elif PROFILE:
+            import profile
+            profile.runctx("main(module_filter, test_filter, libdir)",
+                           globals=globals(), locals=vars(), 
+                           filename=".profile")
+
+        elif HOTSHOT:
+            import hotshot
+            profile = hotshot.Profile(".hotshot")
+            profile.runctx("main(module_filter, test_filter, libdir)",
+                           globals=globals(), locals=vars())
+
         else:
             bad = main(module_filter, test_filter, libdir)
             if bad:
